<?php
declare(strict_types=1);

namespace FiloBlu\Refilo\Helper;

use Magento\Framework\App\ResourceConnection;

/**
 * Class Indexer
 * @package FiloBlu\Refilo\Helper
 */
class Indexer
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * Indexer constructor.
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(ResourceConnection $resourceConnection)
    {
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * @param $table
     * @param $idColumn
     * @return array
     */
    public function getDeletedEntityIds($table, $idColumn)
    {
        return $this->getDeletedEntityIdsByIntervals($this->getDeletedEntityIntervals($table, $idColumn));
    }

    /**
     * @param array $intervals
     * @return array
     */
    public function getDeletedEntityIdsByIntervals(array $intervals)
    {
        $ids = [];

        $end = null;
        foreach ($intervals as $interval) {
            if ($end === null) {
                $end = $interval['end'];
                continue;
            }

            $ids = array_merge($ids, range($end + 1, $interval['start'] - 1));
            $end = $interval['end'];
        }

        return $ids;
    }

    /**
     * @param $table
     * @param $idColumn
     * @return array
     */
    public function getDeletedEntityIntervals($table, $idColumn)
    {
        return $this->resourceConnection->getConnection()->fetchAll
        (
            "SELECT l.{$idColumn} AS `start`, (
                    SELECT MIN(a.{$idColumn}) AS id
                        FROM {$table} AS a
                            LEFT OUTER JOIN {$table} AS b ON a.{$idColumn} = b.{$idColumn} - 1
                        WHERE b.{$idColumn} IS NULL
                            AND a.{$idColumn} >= l.{$idColumn}
                    ) AS `end`
                    FROM {$table} AS l
                        LEFT OUTER JOIN {$table} AS r ON r.{$idColumn} = l.{$idColumn} - 1
                    WHERE r.{$idColumn} IS NULL"
        );
    }
}
